@@ -21,19 +21,26 @@ from typing import TYPE_CHECKING, Any
 
 from ansible.errors import AnsibleError
 from ansible.parsing.splitter import split_args
-from ansible.parsing.yaml.constructor import AnsibleMapping
 from ansible.plugins.loader import add_all_plugin_dirs
 from ansible_compat.runtime import AnsibleWarning
+from ruamel.yaml.parser import ParserError as RuamelParserError
+from yaml.parser import ParserError
+from yaml.scanner import ScannerError
 
 import ansiblelint.skip_utils
 import ansiblelint.utils
-from ansiblelint.app import App, get_app
 from ansiblelint.constants import States
 from ansiblelint.errors import LintWarning, MatchError, WarnSource
-from ansiblelint.file_utils import Lintable, expand_dirs_in_lintables
+from ansiblelint.file_utils import (
+    Lintable,
+    expand_dirs_in_lintables,
+    expand_paths_vars,
+    normpath,
+)
 from ansiblelint.logger import timed_info
 from ansiblelint.rules.syntax_check import OUTPUT_PATTERNS
 from ansiblelint.text import strip_ansi_escape
+from ansiblelint.types import AnsibleJSON, AnsibleMapping
 from ansiblelint.utils import (
     PLAYBOOK_DIR,
     HandleChildren,
@@ -45,6 +52,7 @@ if TYPE_CHECKING:
     from collections.abc import Callable, Generator
 
     from ansiblelint._internal.rules import BaseRule
+    from ansiblelint.app import App
     from ansiblelint.config import Options
     from ansiblelint.constants import FileType
     from ansiblelint.rules import RulesCollection
@@ -106,12 +114,12 @@ class Runner:
             checked_files = set()
         self.checked_files = checked_files
 
-        self.app = get_app(cached=True)
+        self.app = self.rules.app
 
     def _update_exclude_paths(self, exclude_paths: list[str]) -> None:
         if exclude_paths:
             # These will be (potentially) relative paths
-            paths = ansiblelint.file_utils.expand_paths_vars(exclude_paths)
+            paths = expand_paths_vars(exclude_paths)
             # Since ansiblelint.utils.find_children returns absolute paths,
             # and the list of files we create in `Runner.run` can contain both
             # relative and absolute paths, we need to cover both bases.
@@ -166,26 +174,24 @@ class Runner:
                 # For the moment we are ignoring deprecation warnings as Ansible
                 # modules outside current content can generate them and user
                 # might not be able to do anything about them.
-                if warn.category is DeprecationWarning:
+                if warn.category is DeprecationWarning:  # pragma: no cover
                     continue
                 if warn.category is LintWarning:
-                    filename: None | Lintable = None
                     if isinstance(warn.source, WarnSource):
                         match = MatchError(
                             message=warn.source.message or warn.category.__name__,
                             rule=self.rules["warning"],
-                            filename=warn.source.filename.filename,
+                            lintable=Lintable(warn.source.filename.filename),
                             tag=warn.source.tag,
                             lineno=warn.source.lineno,
                         )
                     else:
-                        filename = warn.source
                         match = MatchError(
                             message=(
                                 warn.message if isinstance(warn.message, str) else "?"
                             ),
                             rule=self.rules["warning"],
-                            filename=str(filename),
+                            lintable=Lintable(str(warn.source)),
                         )
                     matches.append(match)
                     continue
@@ -196,7 +202,7 @@ class Runner:
                     warn.category.__name__,
                     warn.message,
                 )
-        return matches
+        return sorted(matches)
 
     def _run(self) -> list[MatchError]:
         """Run the linting (inner loop)."""
@@ -210,14 +216,33 @@ class Runner:
                 self.lintables.remove(lintable)
                 continue
             if isinstance(lintable.data, States) and lintable.exc:
+                line = 1
+                column = None
+                detail = ""
+                sub_tag = ""
                 lintable.exc.__class__.__name__.lower()
+                message = None
+                if lintable.exc.__cause__ and isinstance(
+                    lintable.exc.__cause__,
+                    ScannerError | ParserError | RuamelParserError,
+                ):
+                    sub_tag = "yaml"
+                    if isinstance(lintable.exc.args, tuple):
+                        message = lintable.exc.args[0]
+                    detail = lintable.exc.__cause__.problem
+                    if lintable.exc.__cause__.problem_mark:
+                        line = lintable.exc.__cause__.problem_mark.line + 1
+                        column = lintable.exc.__cause__.problem_mark.column + 1
+
                 matches.append(
                     MatchError(
                         lintable=lintable,
-                        message=str(lintable.exc),
-                        details=str(lintable.exc.__cause__),
+                        message=message or str(lintable.exc),
+                        details=detail or str(lintable.exc.__cause__),
                         rule=self.rules["load-failure"],
-                        tag=f"load-failure[{lintable.exc.__class__.__name__.lower()}]",
+                        lineno=line,
+                        column=column,
+                        tag=f"load-failure[{sub_tag or lintable.exc.__class__.__name__.lower()}]",
                     ),
                 )
                 lintable.stop_processing = True
@@ -244,7 +269,7 @@ class Runner:
 
             for lintable in self.lintables:
                 if (
-                    lintable.kind not in ("playbook", "role")
+                    lintable.kind not in ("playbook", "role", "pattern")
                     or lintable.stop_processing
                 ):
                     continue
@@ -267,17 +292,32 @@ class Runner:
         # do our processing only when ansible syntax check passed in order
         # to avoid causing runtime exceptions. Our processing is not as
         # resilient to be able process garbage.
-        matches.extend(self._emit_matches(files))
-
+        matches.extend(
+            self._emit_matches([file for file in files if not file.failed()])
+        )
+        # mark failed failed lintables as stop processing in order to avoid
+        # duplicated errors from further processing of the other rules
+        for match in matches:
+            if match.lintable.failed():
+                match.lintable.stop_processing = True
+                # Look into making lintables singletons to avoid having to update them
+                for lintable in self.lintables:
+                    if lintable == match.lintable:
+                        lintable.stop_processing = True
+                        break
         # remove duplicates from files list
-        files = [value for n, value in enumerate(files) if value not in files[:n]]
-
+        files = list(dict.fromkeys(files))
         for file in self.lintables:
-            if file in self.checked_files or not file.kind or file.failed():
+            if (
+                file in self.checked_files
+                or not file.kind
+                or file.failed()
+                or file.stop_processing
+            ):
                 continue
             _logger.debug(
                 "Examining %s of type %s",
-                ansiblelint.file_utils.normpath(file.path),
+                normpath(file.path),
                 file.kind,
             )
 
@@ -293,7 +333,7 @@ class Runner:
 
         return sorted(set(matches))
 
-    # pylint: disable=too-many-locals
+    # pylint: disable=too-many-locals,too-many-statements
     def _get_ansible_syntax_check_matches(
         self,
         lintable: Lintable,
@@ -325,8 +365,12 @@ class Runner:
     - ansible.builtin.import_role:
         name: {lintable.path.expanduser()!s}
 """
-                # pylint: disable=consider-using-with
-                fh = tempfile.NamedTemporaryFile(mode="w", suffix=".yml", prefix="play")
+                fh = tempfile.NamedTemporaryFile(  # noqa: SIM115
+                    mode="w",
+                    suffix=".yml",
+                    prefix="play",
+                    encoding="utf-8",
+                )
                 fh.write(playbook_text)
                 fh.flush()
                 playbook_path = fh.name
@@ -337,9 +381,10 @@ class Runner:
             # [WARNING]: provided hosts list is empty, only localhost is available. Note that the implicit localhost does not match 'all'
             cmd = [
                 "ansible-playbook",
-                "-i",
-                "localhost,",
                 "--syntax-check",
+                "-vv",  # needed or ansible-core will fail to mention the loaded file with includes:
+                # statically imported: /foo/bar/malformed.yml
+                # ERROR! A malformed block was encountered while loading a block. ..."
                 playbook_path,
             ]
             if app.options.extra_vars:
@@ -354,11 +399,11 @@ class Runner:
             # https://github.com/ansible/ansible-lint/issues/3650
             env["ANSIBLE_INVENTORY_ANY_UNPARSED_IS_FAILED"] = "False"
 
-            run = subprocess.run(
+            run = subprocess.run(  # noqa: S603
                 cmd,
                 stdin=subprocess.PIPE,
                 capture_output=True,
-                shell=False,  # needed when command is a list # noqa: S603
+                shell=False,  # needed when command is a list
                 text=True,
                 check=False,
                 env=env,
@@ -375,7 +420,7 @@ class Runner:
             stdout = strip_ansi_escape(run.stdout)
             if stderr:
                 details = stderr
-                if stdout:
+                if stdout:  # pragma: no cover
                     details += "\n" + stdout
             else:
                 details = stdout
@@ -427,6 +472,7 @@ class Runner:
                     f"Unexpected error code {run.returncode} from "
                     f"execution of: {' '.join(cmd)}"
                 )
+                filename.failed()
                 results.append(
                     MatchError(
                         message=message,
@@ -455,6 +501,11 @@ class Runner:
         visited: set[Lintable] = set()
         while visited != self.lintables:
             for lintable in self.lintables - visited:
+                visited.add(lintable)
+                if lintable.failed():
+                    continue
+                if not lintable.path.exists():
+                    continue
                 try:
                     children = self.find_children(lintable)
                     for child in children:
@@ -468,16 +519,19 @@ class Runner:
                     exc.rule = self.rules["load-failure"]
                     yield exc
                 except AttributeError:
-                    yield MatchError(lintable=lintable, rule=self.rules["load-failure"])
-                visited.add(lintable)
+                    yield MatchError(
+                        lintable=lintable,
+                        rule=self.rules["load-failure"],
+                    )
 
     def find_children(self, lintable: Lintable) -> list[Lintable]:
         """Traverse children of a single file or folder."""
+        playbook_ds: AnsibleJSON
         if not lintable.path.exists():
             return []
         playbook_dir = str(lintable.path.parent)
         ansiblelint.utils.set_collections_basedir(lintable.path.parent)
-        add_all_plugin_dirs(playbook_dir or ".")
+        add_all_plugin_dirs(playbook_dir or ".")  # type: ignore[no-untyped-call]
         if lintable.kind == "role":
             playbook_ds = AnsibleMapping({"roles": [{"role": str(lintable.path)}]})
         elif lintable.kind == "plugin":
@@ -488,10 +542,9 @@ class Runner:
             try:
                 playbook_ds = ansiblelint.utils.parse_yaml_from_file(str(lintable.path))
             except AnsibleError as exc:
-                msg = f"Loading {lintable.filename} caused an {type(exc).__name__} exception: {exc}, file was ignored."
-                logging.exception(msg)
-                # raise SystemExit(exc) from exc
-                return []
+                raise MatchError(
+                    lintable=lintable, rule=self.rules["load-failure"]
+                ) from exc
         results = []
         # playbook_ds can be an AnsibleUnicode string, which we consider invalid
         if isinstance(playbook_ds, str):
@@ -499,7 +552,7 @@ class Runner:
         for item in ansiblelint.utils.playbook_items(playbook_ds):
             # if lintable.kind not in ["playbook"]:
             for child in self.play_children(
-                lintable.path.parent,
+                lintable,
                 item,
                 lintable.kind,
                 playbook_dir,
@@ -512,7 +565,7 @@ class Runner:
                 # Repair incorrect paths obtained when old syntax was used, like:
                 # - include: simpletask.yml tags=nginx
                 valid_tokens = []
-                for token in split_args(path_str):
+                for token in split_args(path_str):  # type: ignore[no-untyped-call]
                     if "=" in token:
                         break
                     valid_tokens.append(token)
@@ -525,19 +578,19 @@ class Runner:
 
     def play_children(
         self,
-        basedir: Path,
+        lintable: Lintable,
         item: tuple[str, Any],
         parent_type: FileType,
         playbook_dir: str,
     ) -> list[Lintable]:
         """Flatten the traversed play tasks."""
         # pylint: disable=unused-argument
-
+        basedir = lintable.path.parent
         handlers = HandleChildren(self.rules, app=self.app)
 
         delegate_map: dict[
             str,
-            Callable[[str, Any, Any, FileType], list[Lintable]],
+            Callable[[Lintable, Any, Any, FileType], list[Lintable]],
         ] = {
             "tasks": handlers.taskshandlers_children,
             "pre_tasks": handlers.taskshandlers_children,
@@ -545,8 +598,8 @@ class Runner:
             "block": handlers.taskshandlers_children,
             "include": handlers.include_children,
             "ansible.builtin.include": handlers.include_children,
-            "import_playbook": handlers.include_children,
-            "ansible.builtin.import_playbook": handlers.include_children,
+            "import_playbook": handlers.import_playbook_children,
+            "ansible.builtin.import_playbook": handlers.import_playbook_children,
             "roles": handlers.roles_children,
             "dependencies": handlers.roles_children,
             "handlers": handlers.taskshandlers_children,
@@ -556,7 +609,7 @@ class Runner:
             "ansible.builtin.import_tasks": handlers.include_children,
         }
         (k, v) = item
-        add_all_plugin_dirs(str(basedir.resolve()))
+        add_all_plugin_dirs(str(basedir.resolve()))  # type: ignore[no-untyped-call]
 
         if k in delegate_map and v:
             v = template(
@@ -565,7 +618,7 @@ class Runner:
                 {"playbook_dir": PLAYBOOK_DIR or str(basedir.resolve())},
                 fail_on_undefined=False,
             )
-            return delegate_map[k](str(basedir), k, v, parent_type)
+            return delegate_map[k](lintable, k, v, parent_type)
         return []
 
     def plugin_children(self, lintable: Lintable) -> list[Lintable]:
@@ -584,9 +637,10 @@ class Runner:
         examples.line_offset = offset
 
         # pylint: disable=consider-using-with
-        examples.file = NamedTemporaryFile(
+        examples.file = NamedTemporaryFile(  # noqa: SIM115
             mode="w+",
             suffix=f"_{lintable.path.name}.yaml",
+            encoding="utf-8",
         )
         examples.file.write(content)
         examples.file.flush()
@@ -618,14 +672,14 @@ def threads() -> int:
     if os.path.exists(cpu_max_fname):
         # cgroup v2
         # https://www.kernel.org/doc/html/latest/admin-guide/cgroup-v2.html
-        with open(cpu_max_fname, encoding="utf-8") as fh:
+        with open(cpu_max_fname, encoding="utf-8") as fh:  # pragma: no cover
             cpu_quota_us, cpu_period_us = fh.read().strip().split()
     elif os.path.exists(cfs_quota_fname) and os.path.exists(cfs_period_fname):
         # cgroup v1
         # https://www.kernel.org/doc/html/latest/scheduler/sched-bwc.html#management
-        with open(cfs_quota_fname, encoding="utf-8") as fh:
+        with open(cfs_quota_fname, encoding="utf-8") as fh:  # pragma: no cover
             cpu_quota_us = fh.read().strip()
-        with open(cfs_period_fname, encoding="utf-8") as fh:
+        with open(cfs_period_fname, encoding="utf-8") as fh:  # pragma: no cover
             cpu_period_us = fh.read().strip()
     else:
         # No Cgroup CPU bandwidth limit (e.g. non-Linux platform)
