@@ -1,5 +1,6 @@
 """Utility helpers to simplify working with yaml-based data."""
 
+# cspell: ignore docinfo
 # pylint: disable=too-many-lines
 from __future__ import annotations
 
@@ -7,7 +8,7 @@ import functools
 import logging
 import os
 import re
-from collections.abc import Callable, Iterator, Sequence
+from collections.abc import Mapping
 from io import StringIO
 from pathlib import Path
 from re import Pattern
@@ -17,6 +18,7 @@ import ruamel.yaml.events
 from ruamel.yaml.comments import CommentedMap, CommentedSeq, Format
 from ruamel.yaml.composer import ComposerError
 from ruamel.yaml.constructor import RoundTripConstructor
+from ruamel.yaml.docinfo import Version
 from ruamel.yaml.emitter import Emitter, ScalarAnalysis
 
 # Module 'ruamel.yaml' does not explicitly export attribute 'YAML'; implicit reexport disabled
@@ -28,15 +30,25 @@ from yamllint.config import YamlLintConfig
 
 from ansiblelint.constants import (
     ANNOTATION_KEYS,
+    LINE_NUMBER_KEY,
     NESTED_TASK_KEYS,
     PLAYBOOK_TASK_KEYWORDS,
 )
 from ansiblelint.utils import Task
 
+try:  # ansible 2.19 + data tagging
+    # cspell: ignore datatag
+    from ansible._internal._datatag._tags import (  # type: ignore[import-not-found] # pyright: ignore[reportMissingImports]
+        Origin,
+    )
+except ImportError:  # pragma: no cover
+    Origin = None
+
 if TYPE_CHECKING:
     # noinspection PyProtectedMember
+    from collections.abc import Callable, Iterator, Sequence
+
     from ruamel.yaml.comments import LineCol
-    from ruamel.yaml.compat import StreamTextType
     from ruamel.yaml.nodes import ScalarNode
     from ruamel.yaml.representer import RoundTripRepresenter
     from ruamel.yaml.tokens import CommentToken
@@ -46,7 +58,7 @@ if TYPE_CHECKING:
 _logger = logging.getLogger(__name__)
 
 
-class CustomYamlLintConfig(YamlLintConfig):  # type: ignore[misc]
+class CustomYamlLintConfig(YamlLintConfig):
     """Extension of YamlLintConfig."""
 
     def __init__(
@@ -55,7 +67,7 @@ class CustomYamlLintConfig(YamlLintConfig):  # type: ignore[misc]
         file: str | Path | None = None,
     ) -> None:
         """Initialize config."""
-        super().__init__(content, file)
+        super().__init__(content=content, file=file)  # type: ignore[no-untyped-call]
         self.incompatible = ""
 
 
@@ -74,19 +86,25 @@ def deannotate(data: Any) -> Any:
     return data
 
 
-def load_yamllint_config() -> CustomYamlLintConfig:
+def load_yamllint_config(yamllint_file: Path | None = None) -> CustomYamlLintConfig:
     """Load our default yamllint config and any customized override file."""
     config = CustomYamlLintConfig(file=Path(__file__).parent / "data" / ".yamllint")
     config.incompatible = ""
-    # if we detect local yamllint config we use it but raise a warning
+    # Declare local yamllint config file locations.
+    # If we detect local yamllint config we use it but raise a warning
     # as this is likely to get out of sync with our internal config.
-    for path in [
+    yamllint_config_locations = [
         ".yamllint",
         ".yamllint.yaml",
         ".yamllint.yml",
         os.getenv("YAMLLINT_CONFIG_FILE", ""),
         os.getenv("XDG_CONFIG_HOME", "~/.config") + "/yamllint/config",
-    ]:
+    ]
+    if yamllint_file:
+        # Ensure the CLI option yamllint_file config is the first
+        # file to be loaded
+        yamllint_config_locations.insert(0, str(yamllint_file))
+    for path in yamllint_config_locations:
         file = Path(path).expanduser()
         if file.is_file():
             _logger.debug(
@@ -95,7 +113,7 @@ def load_yamllint_config() -> CustomYamlLintConfig:
                 file,
             )
             custom_config = CustomYamlLintConfig(file=str(file))
-            custom_config.extend(config)
+            custom_config.extend(config)  # type: ignore[no-untyped-call]
             config = custom_config
             break
 
@@ -158,7 +176,7 @@ def load_yamllint_config() -> CustomYamlLintConfig:
 
 
 def nested_items_path(
-    data_collection: dict[Any, Any] | list[Any],
+    data_collection: Mapping[Any, Any] | list[Any],
     ignored_keys: Sequence[str] = (),
 ) -> Iterator[tuple[Any, Any, list[str | int]]]:
     """Iterate a nested data structure, yielding key/index, value, and parent_path.
@@ -218,9 +236,9 @@ def nested_items_path(
     """
     # As typing and mypy cannot effectively ensure we are called only with
     # valid data, we better ignore NoneType
-    if data_collection is None:
+    if data_collection is None:  # pragma: no cover
         return
-    data: dict[Any, Any] | list[Any]
+    data: Mapping[Any, Any] | list[Any]
     if isinstance(data_collection, Task):
         data = data_collection.normalized_task
     else:
@@ -233,7 +251,7 @@ def nested_items_path(
 
 
 def _nested_items_path(
-    data_collection: dict[Any, Any] | list[Any],
+    data_collection: Mapping[Any, Any] | list[Any],
     parent_path: list[str | int],
     ignored_keys: Sequence[str] = (),
 ) -> Iterator[tuple[Any, Any, list[str | int]]]:
@@ -245,15 +263,15 @@ def _nested_items_path(
     """
     # we have to cast each convert_to_tuples assignment or mypy complains
     # that both assignments (for dict and list) do not have the same type
-    convert_to_tuples_type = Callable[[], Iterator[tuple[str | int, Any]]]
-    if isinstance(data_collection, dict):
+    # convert_to_tuples_type = Callable[[], Iterator[tuple[str | int, Any]]]
+    if isinstance(data_collection, Mapping):
         convert_data_collection_to_tuples = cast(
-            convert_to_tuples_type,
+            "Callable[[], Iterator[tuple[str | int, Any]]]",
             functools.partial(data_collection.items),
         )
     elif isinstance(data_collection, list):
         convert_data_collection_to_tuples = cast(
-            convert_to_tuples_type,
+            "Callable[[], Iterator[tuple[str | int, Any]]]",
             functools.partial(enumerate, data_collection),
         )
     else:
@@ -284,9 +302,12 @@ def get_path_to_play(
     lc: LineCol  # lc uses 0-based counts
     # lineno is 1-based. Convert to 0-based.
     line_index = lineno - 1
+    if line_index == 0:
+        return []
 
     prev_play_line_index = ruamel_data.lc.line
     last_play_index = len(ruamel_data)
+    play_index = None
     for play_index, play in enumerate(ruamel_data):
         next_play_index = play_index + 1
         if last_play_index > next_play_index:
@@ -295,7 +316,7 @@ def get_path_to_play(
             next_play_line_index = None
 
         lc = play.lc
-        if not isinstance(lc.line, int):
+        if not isinstance(lc.line, int):  # pragma: no cover
             msg = f"expected lc.line to be an int, got {lc.line!r}"
             raise TypeError(msg)
         if lc.line == line_index:
@@ -306,13 +327,15 @@ def get_path_to_play(
         # so, handle the last play separately.
         if (
             next_play_index == last_play_index
-            and line_index > lc.line
+            and line_index <= lc.line
             and (next_play_line_index is None or line_index < next_play_line_index)
         ):
             # part of this (last) play
             return [play_index]
         prev_play_line_index = play.lc.line
-    return []
+    if play_index is None:
+        return []
+    return [play_index]
 
 
 def get_path_to_task(
@@ -325,7 +348,7 @@ def get_path_to_task(
         msg = f"expected lineno >= 1, got {lineno}"
         raise ValueError(msg)
     if lintable.kind in ("tasks", "handlers", "playbook"):
-        if not isinstance(ruamel_data, CommentedSeq):
+        if not isinstance(ruamel_data, CommentedSeq):  # pragma: no cover
             msg = f"expected ruamel_data to be a CommentedSeq, got {ruamel_data!r}"
             raise ValueError(msg)
         if lintable.kind in ("tasks", "handlers"):
@@ -432,7 +455,7 @@ def _get_path_to_task_in_tasks_block(
                 task_path: list[str | int] = [task_index]
                 return task_path + list(subtask_path)
 
-        if not isinstance(task.lc.line, int):
+        if not isinstance(task.lc.line, int):  # pragma: no cover
             msg = f"expected task.lc.line to be an int, got {task.lc.line!r}"
             raise TypeError(msg)
         if task.lc.line == line_index:
@@ -519,7 +542,7 @@ class OctalIntYAML11(ScalarInt):
         return representer.insert_underscore(
             "0",
             v,
-            data._underscore,  # noqa: SLF001
+            data._underscore,
             anchor=anchor,
         )
 
@@ -547,7 +570,7 @@ class CustomConstructor(RoundTripConstructor):
                 underscore = [len(v) - v.rindex("_") - 1, False, False]  # type: Any
             except ValueError:
                 underscore = None
-            except IndexError:
+            except IndexError:  # pragma: no cover
                 underscore = None
             value_s = value_su.replace("_", "")
             if value_s[0] in "+-":
@@ -638,18 +661,33 @@ class FormattedEmitter(Emitter):
         """Select how to quote scalars if needed."""
         style = super().choose_scalar_style()
         if (
-            style == ""
+            style == ""  # noqa: PLC1901
             and self.event.value.startswith("0")
             and len(self.event.value) > 1
         ):
-            if self.event.tag == "tag:yaml.org,2002:int" and self.event.implicit[0]:
-                if self.event.value.startswith("0x"):
-                    self.event.tag = "tag:yaml.org,2002:str"
-                    return ""
-                # ensures that "0123" string does not lose its quoting
+            # We have an as-yet unquoted token that starts with "0" (but is not itself the digit 0).
+            # It could be:
+            # - hexadecimal like "0xF1"; comes tagged as int. Should continue unquoted to continue as an int.
+            # - octal like "0666" or "0o755"; comes tagged as str. **Should** be quoted to be cross-YAML compatible.
+            # - string like "0.0.0.0" and "00-header". Should not be quoted, unless it has a quote in it.
+            if (
+                self.event.value.startswith("0x")
+                and self.event.tag == "tag:yaml.org,2002:int"
+                and self.event.implicit[0]
+            ):
+                # hexadecimal
+                self.event.tag = "tag:yaml.org,2002:str"
+                return ""
+            try:
+                int(self.event.value, 8)
+            except ValueError:
+                pass
+                # fallthrough to string
+            else:
+                # octal
                 self.event.tag = "tag:yaml.org,2002:str"
                 self.event.implicit = (True, True, True)
-            return '"'
+                return '"'
         if style != "'":
             # block scalar, double quoted, etc.
             return style
@@ -715,10 +753,10 @@ class FormattedEmitter(Emitter):
             if "#" in string:
                 # # is \uFF03 (fullwidth number sign)
                 # ﹟ is \uFE5F (small number sign)
-                string = string.replace("#", "\uFF03#\uFE5F")
+                string = string.replace("#", "\uff03#\ufe5f")
                 # this is safe even if this sequence is present
                 # because it gets reversed in post-processing
-        except (ValueError, TypeError):
+        except (ValueError, TypeError):  # pragma: no cover
             # probably not really a string. Whatever.
             pass
         return string
@@ -727,10 +765,10 @@ class FormattedEmitter(Emitter):
     def drop_octothorpe_protection(string: str) -> str:
         """Remove string protection of "#" after full-line-comment post-processing."""
         try:
-            if "\uFF03#\uFE5F" in string:
+            if "\uff03#\ufe5f" in string:
                 # # is \uFF03 (fullwidth number sign)
                 # ﹟ is \uFE5F (small number sign)
-                string = string.replace("\uFF03#\uFE5F", "#")
+                string = string.replace("\uff03#\ufe5f", "#")
         except (ValueError, TypeError):
             # probably not really a string. Whatever.
             pass
@@ -805,6 +843,16 @@ class FormattedEmitter(Emitter):
 class FormattedYAML(YAML):
     """A YAML loader/dumper that handles ansible content better by default."""
 
+    default_config = {
+        "explicit_start": True,
+        "explicit_end": False,
+        "width": 160,
+        "indent_sequences": True,
+        "preferred_quote": '"',
+        "min_spaces_inside": 0,
+        "max_spaces_inside": 1,
+    }
+
     def __init__(  # pylint: disable=too-many-arguments
         self,
         *,
@@ -813,6 +861,7 @@ class FormattedYAML(YAML):
         output: Any = None,
         plug_ins: list[str] | None = None,
         version: tuple[int, int] | None = None,
+        config: dict[str, bool | int | str] | None = None,
     ):
         """Return a configured ``ruamel.yaml.YAML`` instance.
 
@@ -873,7 +922,7 @@ class FormattedYAML(YAML):
                 - name: Task
         """
         if version:
-            if isinstance(version, str):
+            if isinstance(version, str):  # pragma: no cover
                 x, y = version.split(".", maxsplit=1)
                 version = (int(x), int(y))
             self._yaml_version_default: tuple[int, int] = version
@@ -882,17 +931,18 @@ class FormattedYAML(YAML):
 
         # NB: We ignore some mypy issues because ruamel.yaml typehints are not great.
 
-        config = self._defaults_from_yamllint_config()
+        if not config:
+            config = self._defaults_from_yamllint_config()
 
         # these settings are derived from yamllint config
         self.explicit_start: bool = config["explicit_start"]  # type: ignore[assignment]
         self.explicit_end: bool = config["explicit_end"]  # type: ignore[assignment]
         self.width: int = config["width"]  # type: ignore[assignment]
-        indent_sequences: bool = cast(bool, config["indent_sequences"])
-        preferred_quote: str = cast(str, config["preferred_quote"])  # either ' or "
+        indent_sequences: bool = cast("bool", config["indent_sequences"])
+        preferred_quote: str = cast("str", config["preferred_quote"])  # either ' or "
 
-        min_spaces_inside: int = cast(int, config["min_spaces_inside"])
-        max_spaces_inside: int = cast(int, config["max_spaces_inside"])
+        min_spaces_inside: int = cast("int", config["min_spaces_inside"])
+        max_spaces_inside: int = cast("int", config["max_spaces_inside"])
 
         self.default_flow_style = False
         self.compact_seq_seq = True  # type: ignore[assignment] # dash after dash
@@ -927,6 +977,7 @@ class FormattedYAML(YAML):
         # This will only preserve quotes for strings read from the file.
         # anything modified by the transform will use no quotes, preferred_quote,
         # or the quote that results in the least amount of escaping.
+        self.preserve_quotes = True
 
         # If needed, we can use this to change null representation to be explicit
         # (see https://stackoverflow.com/a/44314840/1134951)
@@ -935,15 +986,8 @@ class FormattedYAML(YAML):
     @staticmethod
     def _defaults_from_yamllint_config() -> dict[str, bool | int | str]:
         """Extract FormattedYAML-relevant settings from yamllint config if possible."""
-        config = {
-            "explicit_start": True,
-            "explicit_end": False,
-            "width": 160,
-            "indent_sequences": True,
-            "preferred_quote": '"',
-            "min_spaces_inside": 0,
-            "max_spaces_inside": 1,
-        }
+        config = FormattedYAML.default_config
+
         for rule, rule_config in load_yamllint_config().rules.items():
             if not rule_config:
                 # rule disabled
@@ -968,7 +1012,7 @@ class FormattedYAML(YAML):
                 # one of: bool, "whatever", "consistent"
                 # so, we use True for "whatever" and "consistent"
                 config["indent_sequences"] = bool(indent_sequences)
-            elif rule == "quoted-strings":
+            elif rule == "quoted-strings":  # pragma: no cover
                 quote_type = rule_config["quote-type"]
                 # one of: single, double, any
                 if quote_type == "single":
@@ -976,7 +1020,7 @@ class FormattedYAML(YAML):
                 elif quote_type == "double":
                     config["preferred_quote"] = '"'
 
-        return cast(dict[str, bool | int | str], config)
+        return cast("dict[str, bool | int | str]", config)
 
     @property
     def version(self) -> tuple[int, int] | None:
@@ -992,22 +1036,30 @@ class FormattedYAML(YAML):
         return None
 
     @version.setter
-    def version(self, value: tuple[int, int] | None) -> None:
+    def version(self, val: str | tuple[int, int] | list[int] | Version | None) -> None:
         """Ensure that yaml version uses our default value.
 
         The yaml Reader updates this value based on the ``%YAML`` directive in files.
         So, if a file does not include the directive, it sets this to None.
         But, None effectively resets the parsing version to YAML 1.2 (ruamel's default).
         """
-        if value is not None:
-            self._yaml_version = value
+        if val is not None:
+            if isinstance(val, tuple):
+                self._yaml_version = val
+            elif isinstance(val, list):
+                self._yaml_version = (val[0], val[1])
+            elif isinstance(val, Version):
+                self._yaml_version = (val.major, val.minor)
+            else:
+                msg = f"Unsupported argument {val}"
+                raise TypeError(msg)
         elif hasattr(self, "_yaml_version_default"):
             self._yaml_version = self._yaml_version_default
         # We do nothing if the object did not have a previous default version defined
 
-    def load(self, stream: Path | StreamTextType) -> Any:
+    def load(self, stream: Path | Any) -> Any:
         """Load YAML content from a string while avoiding known ruamel.yaml issues."""
-        if not isinstance(stream, str):
+        if not isinstance(stream, str):  # pragma: no cover
             msg = f"expected a str but got {type(stream)}"
             raise NotImplementedError(msg)
         # As ruamel drops comments for any document that is not a mapping or sequence,
@@ -1047,6 +1099,7 @@ class FormattedYAML(YAML):
         return self._post_process_yaml(
             text,
             strip_version_directive=strip_version_directive,
+            strip_explicit_start=not self.explicit_start,
         )
 
     def _prevent_wrapping_flow_style(self, data: Any) -> None:
@@ -1074,17 +1127,17 @@ class FormattedYAML(YAML):
                 indent += self.sequence_dash_offset
             elif isinstance(parent_key, int):
                 # next level is a sequence
-                indent += cast(int, self.sequence_indent)
+                indent += cast("int", self.sequence_indent)
             elif isinstance(parent_key, str):
                 # next level is a map
-                indent += cast(int, self.map_indent)
+                indent += cast("int", self.map_indent)
 
         if isinstance(key, int) and indent == 0:
             # flow map is an item in a root-level sequence
             indent += self.sequence_dash_offset
         elif isinstance(key, int) and indent > 0:
             # flow map is in a sequence
-            indent += cast(int, self.sequence_indent)
+            indent += cast("int", self.sequence_indent)
         elif isinstance(key, str):
             # flow map is in a map
             indent += len(key + ": ")
@@ -1135,7 +1188,12 @@ class FormattedYAML(YAML):
         return text, "".join(preamble_comments) or None
 
     @staticmethod
-    def _post_process_yaml(text: str, *, strip_version_directive: bool = False) -> str:
+    def _post_process_yaml(
+        text: str,
+        *,
+        strip_version_directive: bool = False,
+        strip_explicit_start: bool = False,
+    ) -> str:
         """Handle known issues with ruamel.yaml dumping.
 
         Make sure there's only one newline at the end of the file.
@@ -1151,6 +1209,10 @@ class FormattedYAML(YAML):
         if strip_version_directive and text.startswith("%YAML"):
             text = text.split("\n", 1)[1]
 
+        # remove explicit document start
+        if strip_explicit_start and text.startswith("---"):
+            text = text.split("\n", 1)[1]
+
         text = text.rstrip("\n") + "\n"
 
         lines = text.splitlines(keepends=True)
@@ -1176,12 +1238,21 @@ class FormattedYAML(YAML):
 
                 full_line_comments.append((i, stripped))
             elif full_line_comments:
+                # match ident of previous non-blank line
+                if not lines[i - 1].lstrip():
+                    prev = lines[i - 2]
+                    space_length = len(prev) - len(prev.lstrip())
+
                 # end of full line comments so adjust to match indent of this line
                 spaces = " " * space_length
                 for index, comment in full_line_comments:
                     lines[index] = spaces + comment
                 full_line_comments.clear()
 
+            if line.rpartition(" ")[2] == "\n":
+                # drop any trailing spaces
+                lines[i] = line.rstrip() + "\n"
+
             cleaned = line.strip()
             if not cleaned.startswith("#") and cleaned.endswith("-"):
                 # got an empty list item. drop any trailing spaces.
@@ -1220,3 +1291,27 @@ def clean_json(
         # neither a dict nor a list, do nothing
         pass
     return obj
+
+
+def get_line_column(data: object, default_line: int = 1) -> tuple[int, int | None]:
+    """Return the line and column of the given data.
+
+    Args:
+        data: Object for which to introspect line number.
+        default_line: fallback default line number to return if no line number is found.
+    """
+    line = 0
+    column = None
+    if isinstance(data, Mapping) and LINE_NUMBER_KEY in data:
+        line = int(data[LINE_NUMBER_KEY])
+    if not line:
+        # ansible 2.19+
+        if Origin:  # pragma: no cover
+            tag = Origin.get_tag(data)
+            line = tag.line_num
+            column = tag.col_num
+        else:  # pre-ansible 2.19
+            if hasattr(data, "ansible_pos"):  # AnsibleUnicode object
+                _, line, column = data.ansible_pos  # pyright: ignore[reportAttributeAccessIssue]
+
+    return (line or default_line, column)
